{ OOP-Deklaration einer Queue                                    <U_Queue.Pas> }
{                                                               vom 09.08.2006 }
{ Dokumentation der Methoden der Klasse TQueue                                 }
{                                                                              }
{ Konstruktor create                                                           }
{ Nachher: Eine leere Schlange ist erzeugt.                                    }
{                                                                               }
{ Anfrage isEmpty: Boolean                                                     }
{ Nachher: Die Anfrage liefert den Wert true, wenn die Schlange keine Elemente }
{          enthlt, sonst liefert sie den Wert false.                          }
{                                                                               }
{ Auftrag enqueue (pObject:TObject)                                            }
{ Vorher:  Die Schlange ist erzeugt.                                           }
{ Nachher: pObject ist als letztes Element in der Schlange abgelegt.           }
{                                                                               }
{ Auftrag dequeue                                                              }
{ Vorher:  Die Schlange ist nicht leer.                                        }
{ Nachher: Das vorderste Element ist aus der Schlange entfernt.                }
{                                                                               }
{ Anfrage front: TObject                                                       }
{ Vorher:  Die Schlange ist nicht leer.                                        }
{ Nachher: Die Anfrage liefert das vorderste Element der Schlange.             }
{          Die Schlange istunverndert.                                        }
{                                                                               }
{ Destruktor destroy                                                           }
{ Nachher: Die Schlange existiert nicht mehr.                                  }
{                                                                              }
{ ----------------------------------------  Version 1.0 (Garmann) 09.08.2006 - }

unit u_queue;

interface

type
     TElement = class
       private
       pNext: TElement;
       pInhalt: TObject;
       public
       constructor create; overload;
       constructor create(Inhalt: TObject); overload;
       function Inhalt: TObject;
       procedure setInhalt(Inhalt: TObject);
       function Next: TElement;
       procedure setNext(Element: TElement);
     end;


     TQueue = class                             { Queue ber Urtyp TElement    }
       private
       pAnfang, pEnde: TElement;                { "Zeiger" auf erstes /        }
                                                {           letztes Element    }
       public
       constructor create;                      { setzt pAnfang, pEnde auf NIL }
       function isEmpty: boolean;               { Queue leer?                  }
       procedure enqueue(pObject: TObject);     { neues Element hinten an Queue}
       procedure dequeue;                       { vorderstes Element lschen   }
       function front: TObject;                 { Rckgabe: vordestes Element  }
       destructor destroy; override;            { lscht alle vorh. Elemente   }
     end;

implementation

{ **** Routinen fr TElement **** }

constructor TElement.create;                { Konstruktor                      }
begin
  inherited create;                         { ererbten Konstruktor aufrufen    }
  pNext:= NIL;                              { und Nachfolger initialisieren    }
end;

constructor TElement.create(Inhalt: TObject);{ Konstruktor mit Inhalt          }
begin
  inherited create;                         { ererbten Konstruktor aufrufen    }
  pNext:= NIL;                              { und Nachfolger initialisieren    }
  pInhalt:= Inhalt;
end;

function TElement.Inhalt: TObject;          { Rckgabe des Inhalts eines Elem. }
begin
  result:= pInhalt;
end;

procedure TElement.setInhalt(Inhalt: TObject); { Setzen des Inhaltsobjekts     }
begin
  pInhalt:= Inhalt;
end;

function TElement.Next: TElement;           { Rckgabe des Nachfolgerelements  }
begin
  result:= pNext;
end;

procedure TElement.setNext(Element: TElement); { Setzen des Nachfolgerelements }
begin
  pNext:= Element;
end;


{ **** Routinen fr TQueue **** }

constructor TQueue.create;                  { Konstruktor                      }
begin
  inherited create;                         { ererbten Konstruktor aufrufen    }
  pAnfang:= nil;                            { und Zeiger auf erstes/letztes El.}
  pEnde:= nil;                              { mit NIL initialisieren           }
end;

destructor TQueue.destroy;                  { Destruktor                       }
begin
  while not isEmpty do dequeue;             { alle Queueelemente lschen       }
  inherited destroy;                        { und schlielich den geerbten     }
                                            { Destruktor aufrufen              }
end;

procedure TQueue.enqueue(pObject: TObject); { Neues element hinten an Queue    }
var e: TElement;
begin
  e:= Telement.create(pObject);
  if isEmpty                                { Wenn Schlange leer ist,          }
    then begin                              {    dann als erstes Element einf. }
           pAnfang:= e;
           pEnde:= e;
         end
    else begin
           pEnde.setNext(e);                {    sonst hinten anfgen          }
           pEnde:= e;
         end;
end;

procedure TQueue.dequeue;                   { Element vorne lschen            }
var h: TElement;
begin
  if not isEmpty                            { nur wenn Queue nicht leer        }
  then begin
         h:= pAnfang;                       { bisheriges erstes Element merken }
         pAnfang:= pAnfang.Next;            { Nachfolger wird erstes Element   }
         h.free;                            { bisheriges erstes Elem. lschen  }
       end;
end;

function TQueue.front: TObject;             { Rckgabe: erstes Element         }
begin
  if isEmpty                                { wenn Stack leer,                 }
  then result:= nil                         {    dann NIL zurckgeben          }
  else result:= pAnfang.Inhalt;             {    sonst erstes Inhaltsobjekt    }
end;

function TQueue.isEmpty: boolean;           { Queue leer?                      }
begin
  result:= (pAnfang=NIL);                   { sAnfang = NIL => Queue leer      }
end;


end.
